@@ -31,12 +31,15 @@ use Lintian::SlidingWindow;
 
 use constant BLOCKSIZE => 16_384;
 
+my $FNAMES = Lintian::Data->new('files/fnames', qr/\s*\~\~\s*/);
 my $FONT_PACKAGES = Lintian::Data->new('files/fonts', qr/\s++/);
 my $TRIPLETS = Lintian::Data->new('files/triplets', qr/\s++/);
 my $LOCALE_CODES = Lintian::Data->new('files/locale-codes', qr/\s++/);
 my $INCORRECT_LOCALE_CODES
   = Lintian::Data->new('files/incorrect-locale-codes', qr/\s++/);
 my $MULTIARCH_DIRS = Lintian::Data->new('common/multiarch-dirs', qr/\s++/);
+my $GENERIC_HEADER_FILES = Lintian::Data->new('files/generic-header-files');
+my $GENERIC_PYTHON_MODULES= Lintian::Data->new('files/generic-python-modules');
 
 my $PRIVACY_BREAKER_WEBSITES= Lintian::Data->new(
     'files/privacy-breaker-websites',
@@ -91,6 +94,10 @@ my $PKG_CONFIG_BAD_REGEX
   = Lintian::Data->new('files/pkg-config-bad-regex',qr/~~~~~/,
     sub { return  qr/$_[0]/xsm;});
 
+my $DOCUMENTATION_FILE_REGEX
+  = Lintian::Data->new('files/documentation-file-regex',
+    qr/~~~~~/,sub { return  qr/$_[0]/xi;});
+
 my $COMPRESS_FILE_EXTENSIONS
   = Lintian::Data->new('files/compressed-file-extensions',
     qr/\s++/,sub { return qr/\Q$_[0]\E/ });
@@ -137,12 +144,19 @@ sub _load_file_package_list_mapping {
         sub {
             my $pkg = strip($_[0]);
             my $pkg_regexp = qr/^$pkg$/x;
-            my $file_regexp = strip($_[1]);
+            my @sliptline = split(/\s*\~\~/, $_[1], 2);
+            my $file_regexp = strip($sliptline[0]);
             $file_regexp =~ s/\$EXT/$ext/g;
+            my $recontents = $reinside;
+            if (scalar(@sliptline) == 2) {
+                my $contents = strip($sliptline[1]);
+                $recontents = qr/$contents/;
+            }
             return {
                 'pkg_re' => $pkg_regexp,
                 'pkg' => $pkg,
                 'match' => qr/$file_regexp/,
+                'contents_re' => $recontents,
             };
         });
     return {
@@ -150,12 +164,11 @@ sub _load_file_package_list_mapping {
         'mapping' => $mapping,
         'ext' => $ext,
         'tag' => $tagname,
-        'reinside' => $reinside,
     };
 }
 
 my $JS_EXT
-  = '(?:(?i)[-._]?(?:compiled|lite|min|pack(?:ed)?|yc)?\.js(?:\.gz)?)$';
+  = '(?:(?i)[-._]?(?:compiled|lite|min|pack(?:ed)?|prod|umd|yc)?\.(js|css)(?:\.gz)?)$';
 my $PHP_EXT = '(?i)\.(?:php|inc|dtd)$';
 my @FILE_PACKAGE_MAPPING = (
     _load_file_package_list_mapping(
@@ -208,13 +221,13 @@ sub _detect_embedded_libraries {
         if($fname =~ m/$typere/) {
             my $mapping = $type->{'mapping'};
             my $typetag = $type->{'tag'};
-            my $reinside = $type->{'reinside'};
           LIBRARY:
             foreach my $library ($mapping->all) {
                 my $library_data = $mapping->value($library);
                 my $mainre = $library_data->{'pkg_re'};
                 my $mainpkg = $library_data->{'pkg'};
                 my $filere = $library_data->{'match'};
+                my $reinside = $library_data->{'contents_re'};
                 unless ($fname =~ m,$filere,) {
                     next LIBRARY;
                 }
@@ -224,9 +237,7 @@ sub _detect_embedded_libraries {
                 if(defined($reinside)) {
                     my $foundre = 0;
                     my $fd = $file->open(':raw');
-
-                    my $sfd
-                      = Lintian::SlidingWindow->new($fd,sub { $_=lc($_); });
+                    my $sfd = Lintian::SlidingWindow->new($fd);
 
                   READWINDOW:
                     while (my $block = $sfd->readwindow) {
@@ -257,7 +268,7 @@ my @flash_nonfree = (
 );
 
 my %PATH_DIRECTORIES = map { $_ => 1 } qw(
-  bin/ sbin/ usr/bin/ usr/sbin usr/games/ );
+  bin/ sbin/ usr/bin/ usr/sbin/ usr/games/ );
 
 # Common files stored in /usr/share/doc/$pkg that aren't sufficient to
 # consider the package non-empty.
@@ -300,8 +311,15 @@ sub run {
     my $pkg_section = $info->field('section', '');
     my $arch = $info->field('architecture', '');
     my $multiarch = $info->field('multi-arch', 'no');
+    my $multiarch_dir = $MULTIARCH_DIRS->value($arch);
     my $ppkg = quotemeta($pkg);
 
+    my %header_dirs = ('usr/include/' => 1);
+    foreach my $x ($MULTIARCH_DIRS->all) {
+        my $dir = $MULTIARCH_DIRS->value($x);
+        $header_dirs{"usr/include/$dir/"} = 1;
+    }
+
     # get the last changelog timestamp
     # if for some weird reasons the timestamp does
     # not exist, it will remain 0
@@ -323,6 +341,8 @@ sub run {
     # Check if package is empty
     my $is_dummy = $info->is_pkg_class('any-meta');
 
+    my $has_sensible_utils = $info->relation('all')->implies('sensible-utils');
+
     # read data from objdump-info file
     foreach my $file (sort keys %{$info->objdump_info}) {
         my $objdump = $info->objdump_info->{$file};
@@ -405,8 +425,10 @@ sub run {
             tag 'file-name-is-not-valid-UTF-8', $file;
         }
 
-        if ($fname =~ m,[*?],) {
-            tag 'file-name-contains-wildcard-character', $file;
+        # check for generic bad filenames
+        foreach my $tag ($FNAMES->all()) {
+            my $regex = $FNAMES->value($tag);
+            tag $tag, $file if $fname =~ m/$regex/;
         }
 
         if ($file->is_hardlink) {
@@ -442,7 +464,7 @@ sub run {
                    $file->gid < 100
                 || $file->gid == 65_534
                 || ($file->gid >= 60_000 && $file->gid < 65_000))
-          ) {
+        ) {
             tag 'wrong-file-owner-uid-or-gid', $file,
               $file->uid . '/' . $file->gid;
         }
@@ -476,7 +498,7 @@ sub run {
             if(index($fname,'oui') > -1 || index($fname,'iab') > -1) {
                 if($fname
                     =~ m,/(?:[^/]-)?(?:oui|iab)(?:\.(txt|idx|db))?(?:\.$COMPRESS_FILE_EXTENSIONS_OR_ALL)?\Z,x
-                  ) {
+                ) {
                     unless ($source_pkg eq 'ieee-data') {
                         tag 'package-installs-ieee-data', $file;
                     }
@@ -509,7 +531,7 @@ sub run {
             # ---------------- /etc/cron.daily, etc.
             elsif ($fname
                 =~ m,^etc/cron\.(?:daily|hourly|monthly|weekly|d)/[^\.].*[\+\.],
-              ) {
+            ) {
                 # NB: cron ships ".placeholder" files, which shouldn't be run.
                 tag 'run-parts-cron-filename-contains-illegal-chars', $file;
             }
@@ -591,8 +613,16 @@ sub run {
         }
         # ---------------- /usr
         elsif ($fname =~ m,^usr/,) {
-            # ---------------- /usr/share/doc
-            if ($fname =~ m,^usr/share/doc/\S,) {
+            # ---------------- /usr/include
+            if ($fname =~ m,^usr/include/,) {
+                if ($file->is_file and $header_dirs{$file->dirname}) {
+                    for my $regex ($GENERIC_HEADER_FILES->all) {
+                        tag 'header-has-overly-generic-name', $fname,
+                          if $file->basename =~ m,$regex,i;
+                    }
+                }
+                # ---------------- /usr/share/doc
+            } elsif ($fname =~ m,^usr/share/doc/\S,) {
                 if ($type eq 'udeb') {
                     tag 'udeb-contains-documentation-file', $file;
                 } else {
@@ -651,9 +681,10 @@ sub run {
 
                     # contains an INSTALL file?
                     if ($fname =~ m,^usr/share/doc/$ppkg/INSTALL(?:\..+)*$,){
-                        tag
-                          'package-contains-upstream-install-documentation',
+                        #<<< No perltidy - tag name too long
+                        tag 'package-contains-upstream-installation-documentation',
                           $file;
+                        #>>>
                     }
 
                     # contains a README for another distribution/platform?
@@ -664,7 +695,7 @@ sub run {
                                |macosx|netbsd|openbsd|osf|redhat|sco|sgi
                                |solaris|suse|sun|vms|win32|win9x|windows
                              )(?:\.txt)?(?:\.gz)?$,xi
-                      ) {
+                    ) {
                         #<<< No tidy (tag name too long)
                         tag 'package-contains-readme-for-other-platform-or-distro',
                           $file;
@@ -683,11 +714,12 @@ sub run {
             }
             # ---------------- arch-indep pkgconfig
             elsif ($file->is_regular_file
-                && $fname
-                =~ m,^usr/(?:lib(/[^/]+)?|share)/pkgconfig/[^/]+\.pc$,) {
-                my $pkg_config_arch = $1 // '';
+                && $fname=~ m,^usr/(lib(/[^/]+)?|share)/pkgconfig/[^/]+\.pc$,){
+                my $prefix = $1;
+                my $pkg_config_arch = $2 // '';
                 $pkg_config_arch =~ s,\A/,,ms;
-
+                tag 'pkg-config-unavailable-for-cross-compilation', $file
+                  if $prefix eq 'lib';
                 my $fd = $file->open(':raw');
                 my $sfd = Lintian::SlidingWindow->new($fd);
               BLOCK:
@@ -741,7 +773,7 @@ sub run {
                 if (   $file->is_file
                     && $fname
                     =~ m,^usr/lib/debug/usr/lib/pyshared/(python\d?(?:\.\d+))/(.++)$,o
-                  ) {
+                ) {
                     my $correct = "usr/lib/debug/usr/lib/pymodules/$1/$2";
                     tag 'python-debug-in-wrong-location', $file, $correct;
                 }
@@ -829,6 +861,9 @@ sub run {
                 } elsif (
                     $dir !~ /^(?:100dpi|75dpi|misc|Type1|encodings|util)$/) {
                     tag 'file-in-unknown-x11-font-directory', $file;
+                } elsif ($file->basename eq 'encodings.dir'
+                    or $file->basename =~ m{fonts\.(dir|scale|alias)}) {
+                    tag 'package-contains-compiled-font-file', $file;
                 }
                 if ($dir =~ /^(?:100dpi|75dpi|misc)$/) {
                     $x11_font_dirs{$dir}++;
@@ -853,7 +888,7 @@ sub run {
                 && $fname !~ m{\A usr/share/p11-kit/modules/
                                   [[:alnum:]][[:alnum:]_.-]*\.module\Z
                               }xsm
-              ) {
+            ) {
                 tag 'incorrect-naming-of-pkcs11-module', $file;
             }
             # ---------------- /usr/share/vim
@@ -892,7 +927,7 @@ sub run {
                         and (  index($block,'flag')>-1
                             or index($block,'/include/') > -1
                             or index($block,'pkg-config')  > -1)
-                      ) {
+                    ) {
                         tag 'old-style-config-script',$file;
                         # could be ok but only if multi-arch: no
                         if($multiarch ne 'no' or $arch eq 'all') {
@@ -942,7 +977,7 @@ sub run {
                                     lib|
                                     local|sbin|share|
                                     src|spool|tmp)/,x
-                  ) {
+                ) {
                     if ($fname =~ m,^usr/lib(?'libsuffix'64|x?32)/,) {
                         my $libsuffix = $+{libsuffix};
                         # eglibc exception is due to FHS. Other are
@@ -956,7 +991,8 @@ sub run {
                             tag 'non-multi-arch-lib-dir', $file;
                         }
                     } else {
-                        tag 'non-standard-dir-in-usr', $file;
+                        tag 'non-standard-dir-in-usr', $file
+                          unless $fname =~ m,^usr/libexec/,; # #834607
                     }
 
                 }
@@ -1007,7 +1043,7 @@ sub run {
                                 |lock|log|opt|run|spool|state
                                 |tmp|www|yp)/
              }xsm
-              ) {
+            ) {
 
                 tag 'non-standard-dir-in-var', $file;
             }
@@ -1068,7 +1104,7 @@ sub run {
                  |mnt|opt|root|run|sbin|srv|sys
                  |tmp|usr|var)  /
           }oxsm
-          ) {
+        ) {
             # Make an exception for the base-files package here and
             # other similar packages because they install a slew of
             # top-level directories for setting up the base system.
@@ -1111,14 +1147,27 @@ sub run {
                 or $fname =~ m,^(?:bin|boot|dev|etc|sbin)/,
                 # non-FHS, but still usual
                 or $fname =~ m,^usr/[^/]+-linux[^/]*/,
+                or $fname =~ m,^usr/libexec/, # FHS 3.0 / #834607
                 or $fname =~ m,^usr/iraf/,
                 # not allowed, but tested individually
                 or $fname =~ m{\A (?:
                         build|home|mnt|opt|root|run|srv
                        |(?:(?:usr|var)/)?tmp)|var/www/}xsm
-              ) {
+            ) {
                 tag 'file-in-unusual-dir', $file;
             }
+
+            if (    defined($multiarch_dir)
+                and $multiarch eq 'foreign'
+                and $fname =~ m,^usr/lib/\Q$multiarch_dir\E/(.*)$,) {
+                my $tail = $1;
+                tag 'multiarch-foreign-cmake-file', $file
+                  if ($tail =~ m,^cmake/.+\.cmake$,);
+                tag 'multiarch-foreign-pkgconfig', $file
+                  if ($tail =~ m,^pkgconfig/[^/]+\.pc$,);
+                tag 'multiarch-foreign-static-library', $file
+                  if ($tail =~ m,^lib[^/]+\.a$,);
+            }
         }
 
         if ($fname =~ m,^(?:usr/)?lib/([^/]+)/$,o) {
@@ -1153,18 +1202,18 @@ sub run {
             && (   $fname =~ m,^usr/lib/python\d+(?:\.\d+/),o
                 || $fname =~ m,^usr/lib/pyshared,o
                 || $fname =~ m,^usr/share/,o)
-          ) {
+        ) {
             tag 'package-installs-python-egg', $file;
         }
 
-        # ---------------- .coverage (coverage.py output)
-        if ($file->basename eq '.coverage') {
-            tag 'package-contains-python-coverage-file', $file;
-        }
-
         # ---------------- .class (compiled Java files)
-        if ($fname =~ m,\.class$,o) {
-            tag 'package-installs-java-bytecode', $file;
+        if (   $fname =~ m,\.class$,o
+            && $fname !~ m,(?:WEB-INF|demo|doc|example|sample|test),o) {
+            my $fd = $file->open;
+            read($fd, my $magic, 4);
+            close($fd);
+            tag 'package-installs-java-bytecode', $file
+              if $magic eq "\xCA\xFE\xBA\xBE";
         }
 
         # ---------------- /usr/lib/site-python
@@ -1220,7 +1269,7 @@ sub run {
                           # ... so they end with ".pl" rather than ".pm"
                           \.pl['"]
                }xsm
-                      ) {
+                    ) {
                         tag 'perl-module-uses-perl4-libs-without-dep',
                           "$file:$. ${1}.pl";
                     }
@@ -1265,7 +1314,7 @@ sub run {
             and not $fname =~ m{ \. (?:
                   # Common "non-license" file extensions...
                    el|[ch]|cc|p[ylmc]|[hu]i|p_hi|html|php|rb|xpm
-                     |png|jpe?g|gif|svg|dtd|mk|lisp|yml
+                     |png|jpe?g|gif|svg|dtd|mk|lisp|yml|rs|ogg
                ) \Z}xsm
             and not $fname=~ m,^usr/share/zope/Products/.*\.(?:dtml|pt|cpt)$,
             and not $fname =~ m,/under\S+License\.docbook$,
@@ -1284,7 +1333,9 @@ sub run {
             # base-files (which is required to ship them)
             and not $fname =~ m,^usr/share/common-licenses/[^/]+$,o
             and not defined $link
-          ) {
+            # Sphinx includes various license files
+            and not $fname =~ m,/_sources/license(\.rst)?\.txt$,oi
+        ) {
 
             # okay, we cannot rule it out based on file name; but if
             # it is an elf or a static library, we also skip it.  (In
@@ -1302,7 +1353,7 @@ sub run {
             # check later to see if it's in a symlinked directory.
             and not $fname =~ m,^usr/share/(?:devhelp/books|gtk-doc/html)/,
             and not $fname =~ m,^usr/share/doc/[^/]+/examples/,
-          ) {
+        ) {
             push(@devhelp, $fname);
         }
 
@@ -1313,6 +1364,9 @@ sub run {
         if ($fname =~ m,/\*\z,) {
             tag 'star-file', $file;
         }
+        if ($fname =~ m,/-\z,) {
+            tag 'hyphen-file', $file;
+        }
 
         # ---------------- misplaced lintian overrides
         if (   $fname =~ m,^usr/share/doc/$ppkg/override\.[lL]intian(?:\.gz)?$,
@@ -1333,7 +1387,7 @@ sub run {
         if (
             $fname =~ m,^usr/share/doc/(?:.+/)?(?:doxygen|html)/
                          .*\.map\.$COMPRESS_FILE_EXTENSIONS_OR_ALL,x
-          ) {
+        ) {
             tag 'file-should-not-be-compressed', $file;
         }
 
@@ -1356,7 +1410,7 @@ sub run {
                   usr/lib/python (\d+(?:\.\d+)?)/
                         (site|dist)-packages/(.++)
         \Z}oxsm
-          ){
+        ){
             my ($debug, $pyver, $loc, $rest) = ($1, $2, $3, $4);
             my ($pmaj, $pmin) = split(m/\./o, $pyver, 2);
             my @correction;
@@ -1392,9 +1446,9 @@ sub run {
             }
             tag 'python-module-in-wrong-location', @correction
               if (@correction);
-            if (    $rest =~ m,^(tests?)(?:\.py|/__init__\.py)$,
-                and $file->is_regular_file) {
-                tag 'python-module-has-overly-generic-name', $fname, "($1)";
+            for my $regex ($GENERIC_PYTHON_MODULES->all) {
+                tag 'python-module-has-overly-generic-name', $fname, "($1)"
+                  if $rest =~ m,^($regex)(?:\.py|/__init__\.py)$,i;
             }
         }
 
@@ -1428,6 +1482,30 @@ sub run {
                 tag 'nfs-temporary-file-in-package', $file;
             }
 
+            # ---------------- using sensible-utils w/o dependency
+            tag 'missing-depends-on-sensible-utils', $file
+              if not $has_sensible_utils
+              and $fname !~ m,^usr/share/(?:doc|locale)/,
+              and $source_pkg ne 'lintian'
+              and detect_sensible_utils($file);
+
+            # ---------------- documentation files
+            unless($fname =~ m,^etc/, or $fname =~ m,^usr/share/doc/,) {
+                foreach my $taboo ($DOCUMENTATION_FILE_REGEX->all) {
+                    my $regex = $DOCUMENTATION_FILE_REGEX->value($taboo);
+                    if($file->basename =~ m{$regex}xi) {
+                        # see #904852
+                        #<<< No perltidy - tag name too long
+                        tag 'package-contains-documentation-outside-usr-share-doc',
+                          $fname
+                          unless $file->basename =~ m{^README}xi
+                          and $file->file_contents =~ m{this directory}xi;
+                        #>>>
+                        last;
+                    }
+                }
+            }
+
             # ---------------- vcs control files
             if (    $fname =~ m,$VCS_FILES_OR_ALL,
                 and $fname !~ m,^usr/share/cargo/registry/,) {
@@ -1448,7 +1526,7 @@ sub run {
                            (?:usr/)?(?:s?bin|games)/[^/]+\.
                            (?:p[ly]|php|rb|[bc]?sh|tcl)
                          \Z}xsm
-              ) {
+            ) {
                 tag 'script-with-language-extension', $file;
             }
 
@@ -1470,7 +1548,11 @@ sub run {
             }
 
             # ---------------- embedded libraries
-            _detect_embedded_libraries($fname, $file, $pkg);
+            _detect_embedded_libraries($fname, $file, $pkg)
+              # Ignore embedded jQuery libraries for Doxygen (#736360)
+              unless $file->basename eq 'jquery.js'
+              and defined(
+                $info->index_resolved_path($file->dirname . 'doxygen.css'));
 
             # ---------------- embedded Feedparser library
             if (    $fname =~ m,/feedparser\.py$,
@@ -1486,7 +1568,7 @@ sub run {
             }
 
             # ---------------- html/javascript
-            if ($fname =~ m,\.(?:x?html?|js|xht|xml|css)$,i) {
+            if ($fname =~ m,\.(?:x?html?\d?|js|xht|xml|css)$,i) {
                 if(     $source_pkg eq 'josm'
                     and $file->basename eq 'defaultpresets.xml') {
                     # false positive
@@ -1495,7 +1577,8 @@ sub run {
                 }
             }
             # ---------------- fonts
-            elsif ($fname =~ m,/([\w-]+\.(?:[to]tf|pfb))$,i) {
+            elsif (
+                $fname =~ m,/([\w-]+\.(?:[to]tf|pfb|woff2?|eot)(?:\.gz)?)$,i) {
                 my $font = lc $1;
                 if (my $font_owner = $FONT_PACKAGES->value($font)) {
                     tag 'duplicate-font-file', "$fname also in", $font_owner
@@ -1503,6 +1586,9 @@ sub run {
                 } elsif ($pkg !~ m/^(?:[ot]tf|t1|x?fonts)-/) {
                     tag 'font-in-non-font-package', $file;
                 }
+                tag 'font-outside-font-dir', $file
+                  unless $fname =~ m,^usr/share/fonts/,;
+
                 my $finfo = $file->file_info;
                 if ($finfo =~ m/PostScript Type 1 font program data/) {
                     my $path = $file->fs_path;
@@ -1514,7 +1600,7 @@ sub run {
                                 $line =~ m{\A [%\s]*
                                    All\s*Rights\s*Reserved\.?\s*
                                        \Z}xsmi
-                              ) {
+                            ) {
                                 #<<< No perltidy - tag name too long
                                 tag 'license-problem-font-adobe-copyrighted-fragment',
                                   $file;
@@ -1529,7 +1615,7 @@ sub run {
                                [%\s]*Copyright\s*\(c\) \s*
                                19\d{2}[\-\s]19\d{2}\s*
                                Adobe\s*Systems\s*Incorporated\.?\s*\Z}xsmi
-                          ) {
+                        ) {
                             $foundadobeline = 1;
                         }
                         # If copy pasted from black book they are
@@ -1711,7 +1797,7 @@ sub run {
                 )
                 and $operm == 01777
                 and $owner eq 'root/root'
-              ) {
+            ) {
                 # actually shipping files here is warned about elsewhere
             } elsif ($fname eq 'usr/src/'
                 and $operm == 02775
@@ -1896,6 +1982,20 @@ sub run {
         }
     }
 
+    # Maintainer scripts
+    open(my $fd, '<', $info->lab_data_path('control-scripts'));
+    while (<$fd>) {
+        m/^\S* (.*)$/
+          or internal_error("bad line in control-scripts file: $_");
+
+        my $file = $info->control_index_resolved_path($1);
+        next if not $file or not $file->is_open_ok;
+
+        tag 'missing-depends-on-sensible-utils', $file
+          if not $has_sensible_utils and detect_sensible_utils($file);
+    }
+    close($fd);
+
     if (!$is_dummy && !$arch_dep_files && $arch ne 'all') {
         tag 'package-contains-no-arch-dependent-files'
           unless $type eq 'udeb';
@@ -2017,17 +2117,25 @@ sub _check_tag_url_privacy_breach {
                       >,xismog;
         my $relcontent = $1;
         if (defined($relcontent)) {
-            if ($relcontent eq 'schema.dct') {
-                # see #736992
-                return;
-            } elsif  ($relcontent eq 'bookmark') {
-                # see #746656
-                return;
-            } elsif ($relcontent eq 'generator-home') {
-                # generator-home is used by texinfo
-                return;
-                # reparse for alternate (css alternate is loaded)
-            } elsif ($relcontent eq 'alternate') {
+            # See, for example, https://www.w3schools.com/tags/att_link_rel.asp
+            my %allowed = (
+                'alternate'         => 1, # #891301
+                'author'            => 1, # #891301
+                'bookmark'          => 1, # #746656
+                'canonical'         => 1, # #762753
+                'copyright'         => 1, # #902919
+                'edituri'           => 1, # #902919
+                'generator'         => 1, # #891301
+                'generator-home'    => 1, # texinfo
+                'help'              => 1, # #891301
+                'license'           => 1, # #891301
+                'next'              => 1, # #891301
+                'prev'              => 1, # #891301
+                'schema.dct'        => 1, # #736992
+                'search'            => 1, # #891301
+            );
+            return if ($allowed{$relcontent});
+            if ($relcontent eq 'alternate') {
                 my $type = $fulltag;
                 $type =~ m,<link
                       (?:\s[^>]+)? \s+
@@ -2052,6 +2160,15 @@ sub _check_tag_url_privacy_breach {
         return;
     }
 
+    # In Mallard XML, <link> is a clickable anchor that will not be
+    # followed automatically.
+    if(    $file->basename =~ '.xml$'
+        && $tagattr eq 'link'
+        && $file->file_contents
+        =~ qr{ xmlns="http://projectmallard\.org/1\.0/"}) {
+        return;
+    }
+
     # track well known site
     foreach my $breaker ($PRIVACY_BREAKER_WEBSITES->all) {
         my $value = $PRIVACY_BREAKER_WEBSITES->value($breaker);
@@ -2070,7 +2187,7 @@ sub _check_tag_url_privacy_breach {
 
     # generic case
     unless (exists $privacybreachhash->{'tag-generic-'.$website}){
-        tag 'privacy-breach-generic', $file, "($url)";
+        tag 'privacy-breach-generic', $file, "[$fulltag] ($url)";
         $privacybreachhash->{'tag-generic-'.$website} = 1;
     }
     return;
@@ -2146,7 +2263,7 @@ sub detect_privacy_breach {
 
     while (my $block = $sfd->readwindow) {
         # Strip comments
-        for my $x (qw(<!--.*?--\s*> /\*.*?\*/)) {
+        for my $x (qw(<!--(?!\[if).*?--\s*> /\*(?!@cc_on).*?\*/)) {
             $block =~ s@$x@@gs;
         }
         # try generic fragment tagging
@@ -2154,11 +2271,12 @@ sub detect_privacy_breach {
             if(index($block,$keyword) > -1) {
                 my $keyvalue = $PRIVACY_BREAKER_FRAGMENTS->value($keyword);
                 my $regex = $keyvalue->{'regex'};
-                if ($block =~ m{$regex}) {
+                if ($block =~ m{($regex)}) {
+                    my $capture = $1;
                     my $breaker_tag = $keyvalue->{'tag'};
                     unless (exists $privacybreachhash{'tag-'.$breaker_tag}){
                         $privacybreachhash{'tag-'.$breaker_tag} = 1;
-                        tag $breaker_tag, $file;
+                        tag $breaker_tag, $file, "(choke on: $capture)";
                     }
                 }
             }
@@ -2167,7 +2285,7 @@ sub detect_privacy_breach {
             qw(src="http src="ftp src="// data-href="http data-href="ftp
             data-href="// codebase="http codebase="ftp codebase="// data="http
             data="ftp data="// poster="http poster="ftp poster="// <link @import)
-          ) {
+        ) {
             next if index($block, $x) == -1;
             detect_generic_privacy_breach($block,\%privacybreachhash,$file);
             last;
@@ -2177,6 +2295,20 @@ sub detect_privacy_breach {
     return;
 }
 
+sub detect_sensible_utils {
+    my ($file) = @_;
+
+    my $fd = $file->open(':raw');
+    my $sfd = Lintian::SlidingWindow->new($fd, sub { $_= lc($_); }, BLOCKSIZE);
+
+    while (my $block = $sfd->readwindow) {
+        return 1
+          if $block=~ m/(?:select-editor|sensible-(?:browser|editor|pager))\b/;
+    }
+    close($fd);
+    return;
+}
+
 1;
 
 # Local Variables:
